#ifndef FENG_H
#define FENG_H 

#include "coord.h"
#include "zbuffer.h"
#include <libprof.h>
#include <stdbool.h>
#include <gint/keyboard.h>
#include <gint/clock.h>
#include <gint/display.h>

/* Coordonnées de la caméra
   on utilise des double pour la direction
   et des int pour la position (bien qu'une copie flottante cachée existe)
*/
extern double FE_dh, FE_dv, FE_roulis;
extern FE_position FE_user;

/* FE_new_frame():
   affiche l'image précedemment dessinée et lance un nouveau cycle de dessin
 
   Dépend de :
      "FxEngine.h"
          FE_move()
      "coord.h"
          FE_set_matrice()
      "zbuffer.h"
          FE_zbuffer_clear()
      <libprof.h>
          prof_init()
          prof_clear()
          prof_enter()
          prof_leave()
          prof_time() */
void FE_new_frame(void); // calls move function

/* FE_get_fps_current():
   obtient le nombre d'Images Par Seconde (IPS) calculé à partir du dernier cycle */
unsigned int  FE_get_fps_current(void);

/* FE_get_fps_min():
   obtient le nombre d'images par seconde le plus bas ayant existé depuis le démarrage du moteur 3d */
unsigned int  FE_get_fps_min(void);

/* FE_get_fps_max():
   obtient le nombre d'images par seconde le plus haut ayant existé depuis le démarrage du moteur 3d */
unsigned int  FE_get_fps_max(void);

/* FE_get_fps_history():
   renvoie, dans une version compactée, les nombres minimum, actuel et maximum d'images par seconde

   Dépend de :
      "FxEngine.h"
         FE_get_fps_min()
         FE_get_fps_max()
         FE_get_fps_max()
      <gint/std/stdio.h>
         sprintf() */
char* FE_get_fps_history(void);


#endif